PROGRAM_NAME='Main, SVSI RMS, Rev 1'

#include 'SNAPI';
(***********************************************************)
(*  FILE CREATED ON: 11/13/2018  AT: 08:00:11              *)
(***********************************************************)
(*  FILE_LAST_MODIFIED_ON: 12/11/2019  AT: 11:52:05        *)
(***********************************************************)

(***********************************************************)
(*          DEVICE NUMBER DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_DEVICE

    dvMASTER		= 0:1:0;
    dvIP_NACT		= 0:2:0;
    
    dvSVSI_ENCODER_1	= 8001:1:0;
    dvSVSI_ENCODER_2	= 8002:1:0;
    dvSVSI_DECODER	= 9001:1:0;
    
    vdvRMS		= 41001:1:0;
    
    

(***********************************************************)
(*               VARIABLE DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_VARIABLE

    volatile char sEncoder1Name[ ] = 'Rack Encoder';
    volatile char sEncoder2Name[ ] = 'Desk Encoder';
    volatile char sEncoderDescription[ ] = 'SVSI 4K Network AV Encoder';
    
    volatile char sDecoderName[ ] = 'Desk Decoder';
    volatile char sDecoderDescription[ ] = 'SVSI 4K Network AV Decoder';

    volatile char sIpAddressNact[] = '239.254.0.1';
    volatile long nIpPortNact = 51004;
 
 
(***********************************************************)
(*                MODULE DEFINITIONS GO BELOW              *)
(***********************************************************)
DEFINE_MODULE 
    'RmsNetLinxAdapter_dr4_0_0' RmsNetLinxAdapter_dr4_0_0(vdvRMS);
    
    'RmsSVSiDeviceMonitor' RmsSVSiDeviceMonitor_1(vdvRMS, dvSVSI_ENCODER_1);
    'RmsSVSiNActMonitor' RmsSVSiNActMonitor_1(vdvRMS, dvSVSI_ENCODER_1, dvIP_NACT);
    'RmsSVSiDeviceMonitor' RmsSVSiDeviceMonitor_2(vdvRMS, dvSVSI_ENCODER_2);
    'RmsSVSiNActMonitor' RmsSVSiNActMonitor_2(vdvRMS, dvSVSI_ENCODER_2, dvIP_NACT);
    'RmsSVSiDeviceMonitor' RmsSVSiDeviceMonitor_3(vdvRMS, dvSVSI_DECODER);
    'RmsSVSiNActMonitor' RmsSVSiNActMonitor_3(vdvRMS, dvSVSI_DECODER, dvIP_NACT);


(***********************************************************)
(*                 STARTUP CODE GOES BELOW                 *)
(***********************************************************)
DEFINE_START

ip_mc_server_open(dvIP_NACT.PORT, sIpAddressNact, nIpPortNact);


#include 'RmsApi';

(***********************************************************)
(*                  THE EVENTS GO BELOW                    *)
(***********************************************************)
DEFINE_EVENT

data_event[vdvRMS]
{
    online:
    {
	stack_var ip_address_struct ipAddress
	
	// Configure Client
	RmsSetDeviceAutoRegister(TRUE);
	get_ip_address(dvMASTER, ipAddress);
	send_command data.device,"'CONFIG.CLIENT.NAME-',ipAddress.HOSTNAME";
	send_command data.device,'CONFIG.CLIENT.ENABLED-true';

	// Configure (optional) NetLinx Devices via vdvRMS proxy commands @HEADER-PARAMS
	// send_command vdvRMS,'@<d:p:s>-RMS-Asset_Name,<name>'
	// send_command vdvRMS,'@<d:p:s>-RMS-Description,<description>'	<- max 160 characters
	send_command data.device,"'@',RmsDevToString(dvSVSI_ENCODER_1),'-RMS-Asset-Name,',sEncoder1Name";
	send_command data.device,"'@',RmsDevToString(dvSVSI_ENCODER_1),'-RMS-Asset-Description,',sEncoderDescription";
	send_command data.device,"'@',RmsDevToString(dvSVSI_ENCODER_2),'-RMS-Asset-Name,',sEncoder2Name";
	send_command data.device,"'@',RmsDevToString(dvSVSI_ENCODER_2),'-RMS-Asset-Description,',sEncoderDescription";
	send_command data.device,"'@',RmsDevToString(dvSVSI_DECODER),'-RMS-Asset-Name,',sDecoderName";
	send_command data.device,"'@',RmsDevToString(dvSVSI_DECODER),'-RMS-Asset-Description,',sDecoderDescription";
    
    
	// Configure N-Act sidecar module
	// Override default monitored properties "'@',RmsDevToString(dvSVSI_DECODER),'-SVSI-NACT-KEYLIST,item|item|..."
	// DEFAULT KEYLIST = 'temperature|fanPercentage|bandWidthMbs'
	send_command data.device,"'@',RmsDevToString(dvSVSI_DECODER),'-SVSI-NACT-KEYLIST,temperature|fanPercentage'"; // Ignore bandwidth at this decoder - not applicable
    }
}

data_event[dvIP_NACT]
{
    online:
    {
	amx_log(AMX_ERROR,"'data_event[dvIP_NACT]:online'");
    }
    offline:
    {
	amx_log(AMX_ERROR,"'data_event[dvIP_NACT]:offline'");
    }
}
    