MODULE_NAME='RmsSVSiNActMonitor'(DEV vdvRMS,
				DEV dvMonitoredDevice,
				DEV dvIpNAct);
#include 'SNAPI';

(***********************************************************)
(*  FILE CREATED ON: 12/10/2019  AT: 11:16:50              *)
(***********************************************************)
(*  FILE_LAST_MODIFIED_ON: 12/11/2019  AT: 11:49:57        *)
(***********************************************************)

(***********************************************************)
(***********************************************************)
// Key Value stuff
DEFINE_CONSTANT
#warn 'Override default keyvalue settings in consideration of named key constants and stored value data requirements' 
    integer TOTAL_KEY_COUNT 	= 10;	
    integer KEY_LENGTH	 	= 25;
    integer VALUE_LENGTH	= 50;
    
#include 'KeyValue';

DEFINE_VARIABLE

    volatile _uKeys uKeys;
(***********************************************************)
(***********************************************************)

(***********************************************************)
(*               CONSTANT DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_CONSTANT

char KEY_IP_ADDRESS[ ] 		= 'ip.address';

integer MAX_KEYS		= 10;


(***********************************************************)
(*               VARIABLE DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_VARIABLE

volatile char assetClientKey[50];

#warn 'Overwrite default sKeyList with send_command vdvRMS,"@,RmsDevToString(dvSVSI_DECODER),-SVSI-NACT-KEYLIST,item|item|...'
volatile char sKeyList[DUET_MAX_CMD_LEN] = 'temperature|fanPercentage|bandWidthMbs';
volatile char sKeyArray[MAX_KEYS][DUET_MAX_HDR_LEN];


#DEFINE INCLUDE_RMS_EVENT_CUSTOM_COMMAND_CALLBACK 
#DEFINE INCLUDE_RMS_EVENT_ASSET_REGISTERED_CALLBACK
#DEFINE INCLUDE_RMS_EVENT_ASSET_PARAM_RESET_CALLBACK
#INCLUDE 'RmsEventListener';

#include 'RmsApi';

define_function integer isInList(char iItem[], char iList[10][])
{
    stack_var integer i

    for(i = max_length_array(iList); i ;i--)
    {
	if(iItem == iList[i])
	{
	    break;
	}
    }
    
    return i;
}

(***********************************************************)
(* Callback from RmsEventListener 			   *)
(* #DEFINE INCLUDE_RMS_EVENT_CUSTOM_COMMAND_CALLBACK       *)
(***********************************************************)
DEFINE_FUNCTION RmsEventCustomCommand(CHAR header[], CHAR data[])
{
    stack_var char iParam[DUET_MAX_PARAM_LEN];
    
    amx_log(AMX_INFO,"'RmsEventCustomCommand(',header,', ',data,')'");
    iParam = DuetParseCmdParam(data);
    if(header[1] == '@')
    {
	header = right_string(header, length_string(header) - 1); // Remove @ prefix
	if(header == RmsDevToString(dvMonitoredDevice) && find_string(iParam, 'SVSI-NACT-KEYLIST', 1))
	{
	    stack_var integer i;

	    sKeyList = DuetParseCmdParam(data);
	    for(i = 1; i <= max_length_array(sKeyArray); i++)
	    {
		sKeyArray[i] = "";
	    }
	    
	    RmsEnumToArray(sKeyList, sKeyArray);
	    if(get_log_level() == AMX_DEBUG)
	    {
		for(i = 1; i <= max_length_array(sKeyArray); i++)
		{
		    send_string 0,"'sKeyArray[',itoa(i),'] = ',sKeyArray[i]";
		}
	    }
	}
    }
}

DEFINE_FUNCTION CHAR IsRmsReadyForParameterUpdates()
{
  RETURN ([vdvRMS,RMS_CHANNEL_ASSETS_REGISTER] && length_string(assetClientKey));
}


#warn 'Wait for Device Monitor to register assets'
DEFINE_FUNCTION RmsEventAssetRegistered(CHAR iClientKey[], LONG assetId, CHAR newAssetRegistration, CHAR registeredAssetDps[])
{
    amx_log(AMX_WARNING,"__FILE__,'--> RmsEventAssetRegistered(registeredAssetDps:',registeredAssetDps,')'");
    if(iClientKey == RmsDevToString(dvMonitoredDevice)) // !! registeredAssetDps is a dynamic_virtual_device to work around db restrictions !!
    {
	assetClientKey = iClientKey;
	RegisterAssetParameters();
	SynchronizeAssetParameters();
    }
}

DEFINE_FUNCTION RmsEventAssetParameterReset(CHAR iClientKey[], CHAR parameterKey[], CHAR parameterName[], CHAR parameterValue[])
{
    amx_log(AMX_WARNING,"__FILE__,'--> RmsEventAssetParameterReset(iClientKey:',iClientKey,')'");
    if(assetClientKey == iClientKey)
    {
	amx_log(AMX_INFO,"__FILE__,'--> parameterKey:',parameterKey,', parameterValue:',parameterValue");
	ResetAssetParameterValue(parameterKey, parameterValue);
    }
}

(***********************************************************)
(* Name:  RegisterAssetParameters                          *)
(* Args:  -none-                                           *)
(*                                                         *)
(* Desc:  This is a callback method that is invoked by     *)
(*        RMS to notify this module that it is time to     *)
(*        register this asset's parameters to be monitored *)
(*        by RMS.                                          *)
(*                                                         *)
(*        This method should not be invoked/called         *)
(*        by any user implementation code.                 *)
(***********************************************************)
DEFINE_FUNCTION RegisterAssetParameters()
{
  // This Duet-based asset monitoring module will
  // automatically register the default monitoring
  // parameters.  If you wish to extend the capabilities
  // and add additional monitored parameters, please
  // add them here.
  if(IsRmsReadyForParameterUpdates())
  {
    stack_var integer i;
    stack_var RmsAssetParameterThreshold threshold;
    
    amx_log(AMX_INFO,"'RegisterAssetParameters(',assetClientKey,')'");
    for(i = 1; i <= max_length_array(sKeyArray); i++) 
    {
	amx_log(AMX_DEBUG,"'sKeyArray[',itoa(i),'] = ',sKeyArray[i]");
	switch(sKeyArray[i])
	{
	    case 'fanPercentage':
	    {
		RmsAssetParameterEnqueueNumber  (
					assetClientKey,
					sKeyArray[i],
					'Fan Speed',
					'Shows the current fan speed at the SVSI endpoint.',
					RMS_ASSET_PARAM_TYPE_NONE,
					atoi(KeyGetValue(uKeys, sKeyArray[i])),
					0,
					10,
					'',
					RMS_ALLOW_RESET_NO,
					0,
					RMS_TRACK_CHANGES_YES
				    );
	    }
	    case 'bandWidthMbs':
	    {
		RmsAssetParameterEnqueueNumberWithBargraph  (
					assetClientKey,
					sKeyArray[i],
					'Bandwidth',
					'Shows the current bandwidth at the SVSI encoder.',
					RMS_ASSET_PARAM_TYPE_NONE,
					atoi(KeyGetValue(uKeys, sKeyArray[i])),
					0,
					1000,
					'Mbs',
					RMS_ALLOW_RESET_NO,
					0,
					RMS_TRACK_CHANGES_YES,
					RMS_ASSET_PARAM_BARGRAPH_GENERAL_PURPOSE
				    );
	    }
	    case 'temperature':
	    {
		RmsAssetParameterEnqueueNumberWithBargraph  (
						    assetClientKey,
						    sKeyArray[i],
						    'CPU Temperature',
						    'Shows the current CPU temperature at the SVSI endpoint.',
						    RMS_ASSET_PARAM_TYPE_TEMPERATURE,
						    atoi(KeyGetValue(uKeys, sKeyArray[i])),
						    20,
						    100,
						    'C',
						    RMS_ALLOW_RESET_NO,
						    0,
						    RMS_TRACK_CHANGES_YES,
						    'cpu.temperature' // Custom bargraph added at the server
						);
						
		threshold.name = 'CPU Temperature';
		threshold.comparisonOperator = RMS_ASSET_PARAM_THRESHOLD_COMPARISON_GREATER_THAN_EQUAL;
		threshold.value = '75';
		threshold.statusType = RMS_STATUS_TYPE_MAINTENANCE;
		threshold.enabled = TRUE;
		threshold.notifyOnRestore = TRUE;
		threshold.notifyOnTrip = TRUE;
		threshold.delayInterval = 30;
		RmsAssetParameterThresholdEnqueueEx(
					    assetClientKey,
					    sKeyArray[i],
					    threshold)
	    }
	}
    }
    // submit all parameter registrations
    RmsAssetParameterSubmit(assetClientKey);
  }
}


(***********************************************************)
(* Name:  SynchronizeAssetParameters                       *)
(* Args:  -none-                                           *)
(*                                                         *)
(* Desc:  This is a callback method that is invoked by     *)
(*        RMS to notify this module that it is time to     *)
(*        update/synchronize this asset parameter values   *)
(*        with RMS.                                        *)
(*                                                         *)
(*        This method should not be invoked/called         *)
(*        by any user implementation code.                 *)
(***********************************************************)
DEFINE_FUNCTION SynchronizeAssetParameters()
{
  // This callback method is invoked when either the RMS server connection
  // has been offline or this monitored device has been offline from some
  // amount of time.   Since the monitored parameter state values could
  // be out of sync with the RMS server, we must perform asset parameter
  // value updates for all monitored parameters so they will be in sync.
  // Update only asset monitoring parameters that may have changed in value.

  // This Duet-based asset monitoring module will
  // automatically synchronize the default monitored
  // parameter values.  If you extended teh monitored
  // parameters with your own custom parameters, please
  // make sure to synchronize those parameter values here.
  if(IsRmsReadyForParameterUpdates())
  {
    stack_var integer i;
    
    for(i = 1; i <= max_length_array(sKeyArray); i++) 
    {
	if(length_string(sKeyArray[i]))
	{
	    if(length_string(keyGetValue(uKeys, sKeyArray[i])))
	    {
		RmsAssetParameterEnqueueSetValue(assetClientKey, sKeyArray[i], keyGetValue(uKeys, sKeyArray[i]));
	    }
	}
	else 
	{
	    break;
	}
    }

    RmsAssetParameterSubmit (assetClientKey);
  }
}



(***********************************************************)
(* Name:  ResetAssetParameterValue                         *)
(* Args:  parameterKey   - unique parameter key identifier *)
(*        parameterValue - new parameter value after reset *)
(*                                                         *)
(* Desc:  This is a callback method that is invoked by     *)
(*        RMS to notify this module that an asset          *)
(*        parameter value has been reset by the RMS server *)
(*                                                         *)
(*        This method should not be invoked/called         *)
(*        by any user implementation code.                 *)
(***********************************************************)
DEFINE_FUNCTION ResetAssetParameterValue(CHAR parameterKey[],CHAR parameterValue[])
{
  // if your monitoring module performs any parameter
  // value tracking, then you may want to update the
  // tracking value based on the new reset value
  // received from the RMS server.
  if(IsRmsReadyForParameterUpdates())
  {
  }
}


(***********************************************************)
(*                 STARTUP CODE GOES BELOW                 *)
(***********************************************************)
DEFINE_START

// Set up default key array
RmsEnumToArray(sKeyList, sKeyArray);


(***********************************************************)
(*                  THE EVENTS GO BELOW                    *)
(***********************************************************)
DEFINE_EVENT

data_event[dvMonitoredDevice]
{
    online:
    {
	stack_var ip_address_struct ipAddress;
	stack_var dev_info_struct iDevInfo;

	get_ip_address(data.device, ipAddress)
	keyAdd(uKeys, KEY_IP_ADDRESS, ipAddress.IPADDRESS);
    }
}

data_event[dvIpNact]
{
    string:
    {
	local_var char iBuffer[DUET_MAX_CMD_LEN];
	
	if(data.sourceip == keyGetValue(uKeys, KEY_IP_ADDRESS))
	{
	    iBuffer = "iBuffer, data.text";
	    amx_log(AMX_WARNING,"'data_event[dvIpNact@',data.sourceip,']{string:',data.text");
	}
	while(find_string(iBuffer, "$0D", 1))
	{
	    stack_var char iCmd[DUET_MAX_CMD_LEN]
	    
	    iCmd = remove_string(iBuffer, "$0D", 1);
	    if(find_string(iCmd,':',1))
	    {
		stack_var char iKey[KEY_LENGTH];
		
		iKey = remove_string(iCmd,':',1);
		set_length_string(iKey, length_string(iKey) - 1);
		if(IsInList(iKey, sKeyArray))
		{
		    iCmd = itoa(atoi(iCmd));  // Remove terminator and force integer value
		    amx_log(AMX_INFO,"'(',iCmd,' != keyGetValue(uKeys, ',iKey,')) = ',RmsBooleanString(iCmd != keyGetValue(uKeys, iKey)),' @ ',assetClientKey");
		    if(iCmd != keyGetValue(uKeys, iKey)) // Only care about +1 degree changes
		    {
			amx_log(AMX_DEBUG,"'keyAdd(uKeys, ',iKey,', ',iCmd,')'");
			keyAdd(uKeys, iKey, iCmd);
			if(IsRmsReadyForParameterUpdates())
			{
			    cancel_wait 'Param Update Debounce'
			    wait 30 'Param Update Debounce'  // Adhere to minimum 3-second server update interval
			    {
				local_var volatile integer i;
				
				amx_log(AMX_WARNING,"'Param Update Debounce Expired -> RmsAssetParameterSetValue(',assetClientKey,')'");
				for(i = 1; i <= max_length_array(sKeyArray); i++) 
				{
				    if(length_string(sKeyArray[i]))
				    {
					if(length_string(keyGetValue(uKeys, sKeyArray[i])))
					{
					    RmsAssetParameterSetValue(assetClientKey, sKeyArray[i], keyGetValue(uKeys, sKeyArray[i]));
					}
				    }
				    else 
				    {
					break;
				    }
				}
			    }
			}
		    }
		}
	    }
	}
    }
}



