$(window).load(function() {
    refreshConfiguration();
    refreshClientVersion();
    refreshClientConnectionStatus();
    setInterval(refreshClientConnectionStatus, 4000);
    
    $("#allowConnectToServerEnabledLabel").click(function() {
        enableConfiguration();
    });
    
    $("#allowConnectToServerDisabledLabel").click(function() {
        disableConfiguration();
    });
    
    $("#serverConfigCancelButton").click(function() {
    	refreshConfiguration();
    	$("#serverConfigCancelButton").blur();
    });
    
    $("#serverConfigTestButton").click(function() {
    	testConfiguration();
    	$("#serverConfigTestButton").blur();
    });
})

var baseURL = "/web/module/RMS/client/api/";
var curConnectionStatusLabelClass = "label-danger";

function updateClientConnectionStatus(connectionStatus, connectionLabelClass) {
    $("#clientConnectionStatus").html(connectionStatus);
    $("#clientConnectionStatus").removeClass(curConnectionStatusLabelClass);
    $("#clientConnectionStatus").addClass(connectionLabelClass);
    curConnectionStatusLabelClass = connectionLabelClass;
}

function refreshClientConnectionStatus() {
    try {
        $.getJSON(baseURL + "client/connection/status", function(data) {
            var connectionStatus = "OFFLINE";
            var connectionStatusLabelClass = "label-danger";
            connectionStatus = data.connectionStatus;
            if (connectionStatus == "INIT") {
                connectionStatus = "INITIALZING";
                connectionStatusLabelClass = "label-default";
            } else if (connectionStatus == "DISABLED") {
                connectionStatusLabelClass = "label-default";
            } else if (connectionStatus == "OFFLINE") {
            	connectionStatus = "CONNECTING IN " + data.delayTimeSeconds + " SECONDS"
                connectionStatusLabelClass = "label-danger";
            } else if (connectionStatus == "CONNECT-SERVER") {
                connectionStatus = "CONNECTING";
                connectionStatusLabelClass = "label-info";
            } else if (connectionStatus == "CONNECT-CLIENT") {
                connectionStatus = "CONNECTING";
                connectionStatusLabelClass = "label-info";
            } else if (connectionStatus == "CONNECT-LOCATION") {
                connectionStatus = "CONNECTING";
                connectionStatusLabelClass = "label-info";
            } else if (connectionStatus == "CONNECT-FAIL") {
                connectionStatus = "CONNECTION FAILURE";
                connectionStatusLabelClass = "label-warning";
            } else if (connectionStatus == "ONLINE") {
                connectionStatusLabelClass = "label-success";
            } else if (connectionStatus == "ONLINE-UNREGISTERED") {
                connectionStatus = "UNREGISTERED";
                connectionStatusLabelClass = "label-warning";
            } else if (connectionStatus == "REINITIALIZE") {
                connectionStatus = "REINITIALIZING";
                connectionStatusLabelClass = "label-info";
            }
            updateClientConnectionStatus(connectionStatus, connectionStatusLabelClass);
        }).error(function() {
            updateClientConnectionStatus("DISCONNECTED", "label-danger");
        })
    } catch (err) {
        updateClientConnectionStatus("DISCONNECTED", "label-danger");
    }
}

function refreshClientVersion() {
    $.getJSON(baseURL + "client/version", function(data) {
        var version = "Version 4";
        version = "Version " + data.clientVersion;
        $("#clientVersion").html(version);
    })
}

function refreshConfiguration() {
    $.getJSON(baseURL + "configuration", function(data) {
        var configuration = data.clientConfiguration;
        if (configuration['client.enabled'] == 'true') {
            $("#allowConnectToServerEnabledLabel").button('toggle');
            enableConfiguration();
        } else {
            $("#allowConnectToServerDisabledLabel").button('toggle');
            disableConfiguration();
        }
        $("#clientGatewayName").val(configuration['client.name'])
        $("#rmsServerUrl").val(configuration['server.url'])
        $("#rmsServerPassword").val(configuration['server.password'])
    })
}
    
function testConfiguration() {
	$("#testConnectionDialogStatus").text("Testing Connection...")
	$("#testConnectionDialogProgressBar").show();
	$("#testConnectionDialogCloseButton").hide();
	$("#myModal").modal({
		keyboard: false
	})
	
	setTimeout(function() {
	    $.getJSON(baseURL + "configuration/test", {
	    	rmsServerUrl: $("#rmsServerUrl").val(),
	    	rmsServerPassword: $("#rmsServerPassword").val()
	    }, function(data) {
	    	$("#testConnectionDialogStatus").text(data.message)
	    	$("#testConnectionDialogProgressBar").hide();
	    	$("#testConnectionDialogCloseButton").show();

	    	//alert("test success: " + data.success + " test message: " + data.message)
	    })
	}, 1000);
}

function enableConfiguration() {
    $("#clientGatewayName").prop('disabled', false);
    $("#rmsServerUrl").prop('disabled', false);
    $("#rmsServerPassword").prop('disabled', false);
    $("#serverConfigTestButton").removeClass('disabled')
}

function disableConfiguration() {
    $("#clientGatewayName").prop('disabled', true);
    $("#rmsServerUrl").prop('disabled', true);
    $("#rmsServerPassword").prop('disabled', true);
    $("#serverConfigTestButton").addClass('disabled')
}
